import os
import socket
import serial.tools.list_ports
import time
import random
import csv
import numpy as np
from colorama import init, Fore, Style


# Παίρνουμε το Workstation ID
workstation_id = socket.gethostname()  # ή os.getenv('COMPUTERNAME')

# Initialize colorama so that text colors reset automatically
init(autoreset=True)

# Control the number of rounds per block (each block: with or without distraction)
ROUNDS_PER_BLOCK = 12 # Adjust this number to control the rounds

def find_arduino():
    ports = serial.tools.list_ports.comports()
    for port in ports:
        if "Arduino" in port.description or "CH340" in port.description or "USB" in port.description:
            return port.device
    return None

# Greek messages (for participant display only) with bright colors
WELCOME_MSG = Fore.LIGHTCYAN_EX + "Καλώς ήρθατε στο πείραμα! Θα δοκιμάσουμε τον χρόνο αντίδρασής σας."
INSTRUCTIONS = Fore.LIGHTYELLOW_EX + """
📌 Οδηγίες:
- Όταν ανάψει το 🟢 (Πράσινο ή κίτρινο LED στο D8), να είστε έτοιμοι.
- Μετά από τυχαίο χρόνο, θα ανάψει το 🔴 (Κόκκινο LED στο D9) που σημαίνει "Πιέστε τώρα".
- Πρέπει να πατήσετε το κουμπί όσο πιο γρήγορα μπορείτε μετά την ένδειξη.
- Σε κάποιους γύρους θα σας εμφανιστεί μια "Διασπαστική Ερώτηση" που θα διαβάζετε.
- Αν δεν πατήσετε το κουμπί μέσα σε 7 δευτερόλεπτα, ο γύρος θα λήξει.
- Αν πατήσετε το κουμπί πριν την ένδειξη (RED), ο γύρος θα ακυρωθεί και θα επαναληφθεί.
"""
ASK_AGE = Fore.LIGHTYELLOW_EX + "Πληκτρολογήστε την ηλικία σας: "
ASK_GENDER = Fore.LIGHTYELLOW_EX + "Πληκτρολογήστε το φύλο σας (M για Άνδρα, F για Γυναίκα): "
END_MSG = Fore.LIGHTCYAN_EX + "Η δοκιμή ολοκληρώθηκε! Δείτε τα αποτελέσματά σας."

DISTRACTING_QUESTIONS = [
    "Παρακαλώ αφιερώστε λίγο χρόνο για να διαβάσετε προσεκτικά την ακόλουθη ερώτηση και, αν το επιθυμείτε, σκεφτείτε την απάντησή σας, ακόμη κι αν δεν απαιτείται άμεση ανταπόκριση: Πώς θα περιέγραφες τις πολυδιάστατες κοινωνικές και τεχνολογικές μεταβολές που έχουν παρατηρηθεί τα τελευταία χρόνια, λαμβάνοντας υπόψη τα θετικά οφέλη αλλά και τις σημαντικές προκλήσεις που αυτές επιφέρουν, επηρεάζοντας βαθιά την καθημερινότητά μας;",
    "Σας παρακαλώ να αφιερώσετε χρόνο για να εξετάσετε προσεκτικά την παρακάτω ερώτηση και να αναλογιστείτε σε βάθος: Με βάση τις παρατηρήσεις σας, πώς πιστεύετε ότι η παγκοσμιοποίηση έχει μετασχηματίσει τις τοπικές κοινωνίες και ποιες συγκεκριμένες προσαρμογές θεωρείτε αναγκαίες για τη διατήρηση και ενίσχυση του πολιτιστικού τους χαρακτήρα;",
    "Παρακαλώ αφιερώστε λίγο χρόνο και εμβαθύνετε στην εξής ερώτηση: Ποιες θεωρείτε ότι είναι οι πιο σημαντικές και πολυσύνθετες προκλήσεις που αντιμετωπίζει το σύγχρονο εκπαιδευτικό σύστημα, και με ποιον τρόπο πιστεύετε ότι οι νέες τεχνολογίες μπορούν να συμβάλουν ουσιαστικά στη συνολική βελτίωση και ανανέωσή του;",
    "Με ιδιαίτερη προσοχή, σας καλώ να εξετάσετε την ακόλουθη ερώτηση: Πώς πιστεύετε ότι η συνεχής εξέλιξη της τεχνολογίας έχει μεταμορφώσει τον τρόπο που επικοινωνούμε, και ποιες βαθύτερες επιπτώσεις θεωρείτε ότι έχει επιφέρει αυτή η εξέλιξη στις διαπροσωπικές σχέσεις και κοινωνικές αλληλεπιδράσεις μας;",
    "Σας παρακαλώ να αφιερώσετε λίγα λεπτά για να διαβάσετε προσεκτικά την παρακάτω ερώτηση και να σκεφτείτε με βάθος: Πώς θα περιγράφατε την πολύπλευρη επίδραση που ασκούν τα μέσα ενημέρωσης στη διαμόρφωση της κοινής γνώμης, και ποιες κρίσιμες αλλαγές θεωρείτε απαραίτητες για την οικοδόμηση ενός πιο ενημερωμένου και διαφανούς μέλλοντος;",
    "Σας ενθαρρύνω να σκεφτείτε βαθιά την ακόλουθη ερώτηση, η οποία ζητά μια πολυεπίπεδη ανάλυση: Με ποιόν τρόπο πιστεύετε ότι οι διάφορες μορφές τέχνης και ο πλούσιος πολιτισμός μπορούν να συμβάλουν ουσιαστικά στη διαμόρφωση μιας κοινωνίας που χαρακτηρίζεται από αυξημένη συνείδηση, ενότητα και κοινωνική αλληλεγγύη;",
    "Παρακαλώ αφιερώστε χρόνο να μελετήσετε προσεκτικά την παρακάτω ερώτηση: Πώς θα περιγράφατε την ταχεία εξέλιξη της παγκόσμιας οικονομίας εν μέσω της συνεχιζόμενης τεχνολογικής επανάστασης, και ποιες συγκεκριμένες στρατηγικές θεωρείτε κρίσιμες για την επίτευξη της βιώσιμης ανάπτυξης σε αυτή τη νέα εποχή;",
    "Σας παρακαλώ να σκεφτείτε προσεκτικά την παρακάτω ερώτηση: Ποια, κατά τη γνώμη σας, είναι τα πλέον επιτακτικά και βασικά περιβαλλοντικά ζητήματα που αντιμετωπίζει ο πλανήτης σήμερα, και ποιος ρόλος πιστεύετε ότι πρέπει να αναλάβει κάθε πολίτης για την προστασία και διατήρηση του φυσικού μας περιβάλλοντος;",
    "Σας καλώ να εξετάσετε με προσοχή την παρακάτω ερώτηση: Με ποιον τρόπο πιστεύετε ότι οι πολιτισμικές διαφορές μπορούν να μετατραπούν σε πηγή δύναμης και έμπνευσης για την ενίσχυση της συνεργασίας μεταξύ διαφορετικών λαών, και ποιοι παράγοντες θεωρείτε ουσιώδεις για την ενίσχυση αυτής της διευρυμένης κατανόησης και αμοιβαίου σεβασμού;",
    "Σας παρακαλώ να αφιερώσετε λίγο χρόνο για να σκεφτείτε προσεκτικά την παρακάτω ερώτηση: Πώς θα περιγράφατε την πολύπλευρη επίδραση που ασκούν τα κοινωνικά μέσα στην ενημέρωση και διαμόρφωση της κοινής γνώμης, και ποια είναι, κατά τη γνώμη σας, τα κυρίαρχα πλεονεκτήματα και τα πιθανά μειονεκτήματα που συνοδεύουν αυτήν την επιρροή στην κοινωνία μας;",
    "Σε μια υποθετική κατάσταση όπου θα μπορούσατε να αποκτήσετε μια μοναδική υπερδύναμη, ποια θα επιλέγατε να είναι αυτή, με ποιον τρόπο θα την ενσωματώνατε στην καθημερινότητά σας και γιατί πιστεύετε ότι η συγκεκριμένη δύναμη θα είχε ουσιαστικό αντίκτυπο τόσο στην προσωπική σας ζωή όσο και στο ευρύτερο κοινωνικό περιβάλλον;",
    "Παρακαλώ αναπτύξτε μια περιγραφή μιας σημαντικής απόφασης που λάβατε στη ζωή σας, εξηγώντας αναλυτικά τα εσωτερικά κίνητρα που επηρέασαν την επιλογή σας, τις συνέπειες που είχε τόσο στις προσωπικές όσο και στις επαγγελματικές σας επιλογές, και τι πολύτιμα μαθήματα αποκομίσατε από αυτήν την εμπειρία που συνεχώς διαμορφώνει τις μελλοντικές σας αποφάσεις;",
    "Μοιραστείτε μαζί μας ποιο θεωρείτε ότι είναι το πιο συναρπαστικό και ενδιαφέρον στοιχείο που μάθατε πρόσφατα, παρέχοντας μια εκτενή ανάλυση που να περιλαμβάνει λεπτομέρειες για το πώς αυτή η γνώση επηρέασε την αντίληψή σας για τον κόσμο και τι νέες προοπτικές έχει ανοίξει για το μέλλον σας;",
    "Φανταστείτε ότι διαθέτετε ένα εκατομμύριο ευρώ: πώς θα το χρησιμοποιούσατε στη ζωή σας; Σας παρακαλώ να αναλύσετε λεπτομερώς τις πιθανές επιλογές σας, είτε πρόκειται για σοβαρές επενδύσεις είτε για φιλανθρωπικές δράσεις, και να εξηγήσετε τους συγκεκριμένους λόγους και τα κίνητρα που θα σας οδήγησαν σε αυτές τις αποφάσεις;",
    "Σε μια υποθετική κατάσταση όπου θα μπορούσατε να πραγματοποιήσετε ταξίδι στο χρόνο, σε ποια ιστορική εποχή ή συγκεκριμένη στιγμή θα προτιμούσατε να μεταβείτε, ποιος θα ήταν ο στόχος σας κατά τη διάρκεια αυτού του ταξιδιού, και γιατί πιστεύετε ότι αυτή η μοναδική εμπειρία θα είχε βαθύ προσωπικό και κοινωνικό αντίκτυπο στη ζωή σας;",
    "Παρακαλώ αφιερώστε λίγο χρόνο για να σκεφτείτε και να περιγράψετε το πιο περίπλοκο πρόβλημα που έχετε αντιμετωπίσει ποτέ, εξηγώντας λεπτομερώς τις διάφορες προκλήσεις που συναντήσατε, τις στρατηγικές και τις μεθόδους που υιοθετήσατε για να το ξεπεράσετε, καθώς και τα σημαντικά μαθήματα ζωής που αποκομίσατε από αυτήν την εμπειρία;",
    "Σε μια υποθετική κατάσταση όπου θα μπορούσατε να επηρεάσετε το παγκόσμιο σκηνικό, ποιο είναι το ένα πράγμα που θα αλλάζατε, ποιοι θετικοί μετασχηματισμοί θα προέκυπταν στην κοινωνία και γιατί θεωρείτε ότι αυτή η αλλαγή είναι απαραίτητη για την επίτευξη ενός πιο δίκαιου και καλύτερου μέλλοντος για όλους;",
    "Αναλογιστείτε και περιγράψτε με αναλυτικές λεπτομέρειες τι αντιπροσωπεύει για εσάς η έννοια της επιτυχίας, αναφέροντας τους βασικούς παράγοντες που θεωρείτε κρίσιμες για την επίτευξή της σε προσωπικό και επαγγελματικό επίπεδο, και εξηγώντας πώς αυτοί οι παράγοντες αντικατοπτρίζουν τις βαθύτερες αξίες και φιλοδοξίες σας;",
    "Σκεφτείτε για λίγο και απαντήστε: Ποια είναι η αξία που θεωρείτε ότι έχει τη μεγαλύτερη σημασία στη ζωή σας, και με ποιον τρόπο έχει διαμορφώσει τις αποφάσεις, τις διαπροσωπικές σχέσεις και τις προσωπικές σας επιδιώξεις; Παρακαλώ αναλύστε με συγκεκριμένα παραδείγματα από τις προσωπικές σας εμπειρίες που καταδεικνύουν την ουσιώδη της σημασία και επιρροή;",
    "Σας παρακαλώ να σκεφτείτε προσεκτικά την ακόλουθη ερώτηση: Πώς θα επιχειρούσατε να εξηγήσετε την έννοια και τη λειτουργία του Διαδικτύου σε κάποιον που ζούσε κατά τον 19ο αιώνα, λαμβάνοντας υπόψη τις ριζικές τεχνολογικές και κοινωνικές διαφορές εκείνης της εποχής, και πώς πιστεύετε ότι θα αντιδρούσε ή θα το ερμηνεύε σε ένα τόσο διαφορετικό ιστορικό πλαίσιο;",
    "Σας καλώ να μοιραστείτε με ειλικρίνεια ποιο θεωρείτε ότι είναι το πιο μεγάλης σημασίας όνειρό σας, εξηγώντας γιατί το θεωρείτε τόσο ουσιώδες, και να αναλύσετε λεπτομερώς τα βήματα που σκοπεύετε να ακολουθήσετε για την επίτευξή του, συμπεριλαμβανομένων των πιθανών προκλήσεων και των στρατηγικών που θα εφαρμόσετε για να μεταμορφώσετε το όνειρό σας σε πραγματικότητα;",    "Τι πιστεύετε ότι θα άλλαζε ο κόσμος αν όλοι οι άνθρωποι στη γη έστελναν καθημερινά ευχαριστήρια μηνύματα στους συνανθρώπους τους; Εξηγήστε τις πιθανές κοινωνικές, ψυχολογικές και πολιτισμικές επιπτώσεις μιας τέτοιας ευγενικής κίνησης.",
    "Πώς ορίζετε την ευτυχία στη ζωή σας; Περιγράψτε τα στοιχεία που πιστεύετε ότι συνθέτουν την αληθινή ευτυχία, και πώς προσπαθείτε να ενσωματώσετε αυτές τις έννοιες στην καθημερινή σας ζωή για να επιτύχετε προσωπική ισορροπία και αρμονία.",
    "Παρακαλώ αναφέρετε τέσσερα από τα πιο αγαπημένα σας φαγητά, εξηγώντας γιατί κάθε ένα από αυτά έχει ξεχωριστή θέση στη ζωή σας και πώς συνδέονται με ιδιαίτερες αναμνήσεις ή εμπειρίες.",
    "Παρακαλώ αναφέρετε τρεις ταινίες που σας έχουν εντυπωσιάσει ιδιαίτερα, εξηγώντας ποια στοιχεία της σκηνοθεσίας, του σεναρίου ή της ερμηνείας σας συγκίνησαν και πώς αυτές οι ταινίες αντικατοπτρίζουν τα προσωπικά σας γούστα.",
    "Παρακαλώ αναφέρετε τέσσερα χρώματα που προτιμάτε, εξηγώντας πώς κάθε ένα από αυτά επηρεάζει τη διάθεσή σας, τις αισθητικές σας προτιμήσεις και αν υπάρχει κάποια ιδιαίτερη ιστορία ή συμβολισμός πίσω από αυτά.",
    "Παρακαλώ αναφέρετε τρεις χώρες που θα θέλατε να επισκεφθείτε στο μέλλον, εξηγώντας τι σας ελκύει σε κάθε μία από αυτές – όπως ο πολιτισμός, τα τοπία, ή η ιστορία – και τι ελπίζετε να ανακαλύψετε μέσα από το ταξίδι σας.",
    "Παρακαλώ αναφέρετε ποια είναι τα τρία αγαπημένα σας φρούτα, εξηγώντας τις γεύσεις, τις υφές και τις προσωπικές σας αναμνήσεις που συνδέονται με αυτά, καθώς και πώς συμβάλλουν στην καθημερινή σας διατροφή και υγεία.",
    "Παρακαλώ αναφέρετε τέσσερα ζώα που σας αρέσουν ή σας εμπνέουν, εξηγώντας τους λόγους πίσω από την προτίμησή σας, είτε λόγω της ομορφιάς, της συμπεριφοράς είτε λόγω των συμβολικών τους χαρακτηριστικών.",
    "Παρακαλώ αναφέρετε τρεις αγαπημένους σας τραγουδιστές ή μουσικούς, εξηγώντας πώς η μουσική τους έχει επηρεάσει τη ζωή σας και ποια χαρακτηριστικά τους κάνουν να ξεχωρίζουν για εσάς.",
    "Παρακαλώ αναφέρετε τέσσερα πράγματα ή δραστηριότητες που σας γεμίζουν βαθιά χαρά και ικανοποίηση, εξηγώντας πώς συμβάλλουν στη βελτίωση της διάθεσής σας και στην προσωπική σας ευεξία.",
    "Παρακαλώ αναφέρετε τρία αθλήματα που απολαμβάνετε να παρακολουθείτε, εξηγώντας τι σας ελκύει σε αυτά – όπως ο ανταγωνισμός, η ομαδικότητα ή η τεχνική δεξιότητα – και πώς αυτά τα αθλήματα αντικατοπτρίζουν τα ενδιαφέροντά σας.",
    "Παρακαλώ αναφέρετε τέσσερις δραστηριότητες ή χόμπι που σας βοηθούν να χαλαρώνετε και να ανανεώνετε την ενέργειά σας, εξηγώντας πώς συμβάλλουν στη διαχείριση του στρες και στη δημιουργία προσωπικής ισορροπίας.",
    "Παρακαλώ αναφέρετε ποιες είναι οι αγαπημένες σας τηλεοπτικές σειρές, εξηγώντας τι σας ελκύει σε αυτές – όπως η πλοκή, οι χαρακτήρες ή η παραγωγή – και πώς αυτές αντικατοπτρίζουν τα ενδιαφέροντα και τις προτιμήσεις σας.",
    "Παρακαλώ αναφέρετε ποιο είναι το αγαπημένο σας βιβλίο, εξηγώντας πώς σας έχει επηρεάσει, τι μαθήματα αποκομίσατε από αυτό, και γιατί θεωρείτε ότι έχει ιδιαίτερη σημασία στη ζωή σας.",
    "Παρακαλώ αναφέρετε δύο από τα πιο αγαπημένα σας παιχνίδια, είτε πρόκειται για βιντεοπαιχνίδια είτε για επιτραπέζια, εξηγώντας ποια στοιχεία τους κάνουν να ξεχωρίζουν για εσάς και πώς αυτές οι δραστηριότητες συμβάλλουν στην ψυχαγωγία σας."
    "Παρακαλώ αναφέρετε με λεπτομέρεια τα τέσσερα πιο αξέχαστα γεύματα που έχετε δοκιμάσει στη ζωή σας, περιγράφοντας όχι μόνο τα φαγητά που απολαύσατε, αλλά και το περιβάλλον στο οποίο βρισκόσασταν, τα άτομα με τα οποία μοιραστήκατε τη στιγμή, καθώς και τα συναισθήματα που σας προκάλεσε η κάθε εμπειρία. Τι έκανε το κάθε γεύμα τόσο ιδιαίτερο και πώς έχει επηρεάσει τον τρόπο που αντιλαμβάνεστε το φαγητό και τις γευστικές εμπειρίες γενικότερα;",  
    "Ποιες είναι οι τρεις ταινίες που θεωρείτε ότι επηρέασαν περισσότερο τη σκέψη και τις αντιλήψεις σας, και εξηγήστε αναλυτικά τι σας άρεσε σε κάθε μία από αυτές. Μπορείτε να περιγράψετε πώς σας έκανε να αισθανθείτε κάθε ταινία, ποια θεματικά ή φιλοσοφικά ζητήματα σας κέντρισαν το ενδιαφέρον, και αν άλλαξε τον τρόπο που βλέπετε συγκεκριμένα θέματα ή καταστάσεις στη ζωή σας;",
    "Μπορείτε να αναφέρετε τέσσερα διαφορετικά χρώματα που σας συνδέονται με ιδιαίτερες αναμνήσεις ή συναισθήματα και να εξηγήσετε γιατί θεωρείτε ότι αυτά έχουν ιδιαίτερη σημασία για εσάς. Περιγράψτε σε ποιες περιστάσεις ή εμπειρίες τα συναντήσατε, αν σας φέρνουν στο μυαλό συγκεκριμένα άτομα, τοποθεσίες ή γεγονότα, καθώς και αν επηρεάζουν τη διάθεσή σας ή τον τρόπο που αντιλαμβάνεστε τον κόσμο γύρω σας;",
    "Παρακαλώ αναφέρετε τρεις χώρες που θα θέλατε να επισκεφθείτε στο μέλλον, διευκρινίζοντας ποιοι πολιτισμικοί ή τοπικοί παράγοντες σας ελκύουν σε κάθε μία από αυτές. Μπορείτε να περιγράψετε συγκεκριμένα μέρη που σας ενδιαφέρουν, τοπικές παραδόσεις, ιστορικά μνημεία ή φυσικά τοπία που θα θέλατε να εξερευνήσετε, καθώς και το τι ελπίζετε να βιώσετε ή να μάθετε από αυτό το ταξίδι;",
    "Ποια είναι τα τρία αγαπημένα σας φρούτα και πώς πιστεύετε ότι συμβάλλουν στην καθημερινή σας ενέργεια και υγεία; Μπορείτε να αναφέρετε συγκεκριμένους λόγους για τους οποίους τα προτιμάτε, όπως η γεύση, η διατροφική τους αξία ή η σύνδεσή τους με παιδικές αναμνήσεις, καθώς και αν τα ενσωματώνετε σε συγκεκριμένες συνήθειες ή ρουτίνες της καθημερινότητάς σας;",
    "Αναφέρετε τέσσερα ζώα που σας εμπνέουν ή σας θυμίζουν σημαντικές στιγμές της ζωής σας, και εξηγήστε γιατί κάθε ένα από αυτά είναι ξεχωριστό για εσάς. Μπορείτε να περιγράψετε αν σχετίζονται με συγκεκριμένες εμπειρίες, αν ταυτίζεστε με κάποια από τα χαρακτηριστικά τους, ή αν συμβολίζουν για εσάς κάτι σημαντικό, όπως δύναμη, σοφία, ελευθερία ή αφοσίωση;",
    "Μπορείτε να ονομάσετε τρεις αγαπημένους καλλιτέχνες ή τραγουδιστές που θεωρείτε ότι έχουν συμβάλει ουσιαστικά στην καλλιτεχνική σας ανάπτυξη και να εξηγήσετε με ποιον τρόπο; Περιγράψτε πώς το έργο τους σας επηρέασε, ποια συναισθήματα ή σκέψεις σας προκαλεί η μουσική τους ή η τέχνη τους, καθώς και αν υπάρχει κάποια συγκεκριμένη στιγμή ή περίοδος της ζωής σας που συνδέεται άμεσα με αυτούς τους δημιουργούς;",
    "Περιγράψτε τέσσερα πράγματα που σας γεμίζουν βαθιά χαρά και ικανοποίηση, αναφέροντας παραδείγματα από προσωπικές εμπειρίες που τα αντικατοπτρίζουν. Μπορεί να είναι δραστηριότητες, σχέσεις, καθημερινές συνήθειες ή συγκεκριμένα γεγονότα που σας κάνουν να νιώθετε πλήρεις και ευτυχισμένοι. Πώς επηρεάζουν τη διάθεσή σας και ποια είναι η σημασία τους στη ζωή σας;",
    "Αναφέρετε τρία αθλήματα που απολαμβάνετε τόσο ως θεατής όσο και ως συμμετέχων, εξηγώντας τι σας ελκύει σε αυτά και ποια αξία τα καθιστά σημαντικά για εσάς. Μπορείτε να αναφερθείτε στη σωματική ή ψυχική ευεξία που σας προσφέρουν, στις δεξιότητες που απαιτούν, στις αναμνήσεις που έχετε από αυτά, καθώς και στο πώς ενισχύουν το αίσθημα της ομαδικότητας, του ανταγωνισμού ή της προσωπικής προόδου;",
    "Μπορείτε να περιγράψετε με λεπτομέρεια τέσσερις δραστηριότητες ή χόμπι που σας βοηθούν να χαλαρώνετε και να ανανεώνετε την ενέργειά σας μετά από μια απαιτητική μέρα; Αναφέρετε γιατί επιλέγετε αυτές τις δραστηριότητες, πώς σας κάνουν να αισθάνεστε, καθώς και αν έχουν κάποια ειδική σημασία για εσάς, όπως η σύνδεση με αγαπημένα πρόσωπα, η δημιουργικότητα ή η ανάγκη για προσωπικό χρόνο;",
]

# Global list to track available distracting questions (to avoid repeats within a session)
available_distracting = DISTRACTING_QUESTIONS.copy()

# --- Function to generate a unique filename ---
def generate_unique_filename(base_name, extension=".csv"):
    # Ο πλήρης έλεγχος γίνεται μόνο για το ακριβές όνομα, π.χ. "data.csv"
    simple_candidate = base_name + extension
    if not os.path.exists(simple_candidate):
        # Αν δεν υπάρχει αρχείο με το απλό όνομα, δημιουργούμε ένα μοναδικό όνομα με τυχαίο αριθμό και χρονική σήμανση
        random_number = f"{random.randint(0, 999999):06d}"
        timestamp = time.strftime("%Y%m%d-%H%M%S")
        return f"{base_name}_{random_number}_{timestamp}{extension}"
    else:
        # Αν υπάρχει ήδη αρχείο με το βασικό όνομα, επιστρέφουμε αυτό το όνομα (χωρίς να προσθέτουμε επιπλέον στοιχεία)
        return simple_candidate

def get_reaction_time(arduino):
    """
    Περιμένει να λάβει χρόνο αντίδρασης από το Arduino.
    Αν λάβει "PREMATURE", επιστρέφει -1 ώστε να επαναληφθεί ο γύρος.
    Λήγει μετά από 7 δευτερόλεπτα αν δεν ληφθεί έγκυρο δεδομένο,
    και σε αυτή την περίπτωση επιστρέφει 7000 ms.
    """
    start_time = time.time()
    while True:
        if arduino.in_waiting > 0:
            try:
                data = arduino.readline().decode("utf-8").strip()
                if data.startswith("PREMATURE"):
                    return -1  # Ειδική τιμή για πρόωρη πίεση
                if data.startswith("REACTION_TIME,"):
                    return int(data.split(",")[1])
            except ValueError:
                continue  # Παράβλεψη λανθασμένων μηνυμάτων
        if time.time() - start_time > 7:
            return 7000


def run_trial(arduino, trial_num, distraction=False):
    """
    Executes a single trial.
    Returns a tuple: (Trial, Reaction_Time, Distraction (yes/no))

    - Displays a warning message before each trial.
    - Adds a countdown before starting.
    - If the trial is aborted due to a premature press (PREMATURE),
      the trial is repeated with a new distracting question (if applicable)
      and a new random time delay.
    """
    while True:  # Keep retrying the trial if necessary
        print(Fore.LIGHTCYAN_EX + f"\n🔹 Γύρος {trial_num}/{ROUNDS_PER_BLOCK*2}")

        if distraction:
            print(Fore.LIGHTRED_EX + "⚠️ Αυτός ο γύρος έχει ερώτηση! Να είστε έτοιμοι να τη διαβάσετε.")
        else:
            print(Fore.LIGHTGREEN_EX + "✅ Αυτός ο γύρος ΔΕΝ έχει ερώτηση. Συγκεντρωθείτε ΜΟΝΟ στο LED.")

        # 🕒 Countdown before starting
        for i in range(5, 0, -1):
            print(Fore.LIGHTMAGENTA_EX + f"⏳ {i}...", end="\n")
            print()
            if distraction:
                print(Fore.LIGHTRED_EX + "⚠️ Αυτός ο γύρος έχει ερώτηση! Να είστε έτοιμοι να τη διαβάσετε.")
            else:
                print(Fore.LIGHTGREEN_EX + "✅ Αυτός ο γύρος ΔΕΝ έχει ερώτηση. Συγκεντρωθείτε ΜΟΝΟ στο LED.")
            # print()
            time.sleep(1)
        print(Fore.LIGHTMAGENTA_EX + "🚀 ΞΕΚΙΝΑΜΕ!")

        print(Fore.LIGHTGREEN_EX + "\n🟢 Ετοιμαστείτε...")
        time.sleep(0.2)

        if distraction:
            global available_distracting
            if available_distracting:
                question = random.choice(available_distracting)
                available_distracting.remove(question)
            else:
                question = random.choice(DISTRACTING_QUESTIONS)

            # 🔹 Add random spaces (between 20 and 50 spaces)
            random_spaces = " " * random.randint(20, 370)
            formatted_question = random_spaces + question
            print(Fore.LIGHTYELLOW_EX + f"⚠️ {formatted_question}")
            time.sleep(0.1)
            distraction_flag = "yes"
        else:
            distraction_flag = "no"

        delay = random.uniform(2.0, 3.8)
        arduino.write(f"START,{delay}\n".encode("utf-8"))
        reaction_time = get_reaction_time(arduino)

        if reaction_time == -1:  # Handle premature button press
            print(Fore.LIGHTRED_EX + "❌ Πρόωρη πίεση! Ο γύρος θα επαναληφθεί.")
            time.sleep(1)  # Short delay before retrying
            continue  # Restart the trial
        
        # If valid reaction time is received, record and exit loop
        return (trial_num, reaction_time, distraction_flag)

def process_results(user_id, age, gender, distraction_sequence, results):
    """
    Saves detailed and summary results after the assistant approves the experiment.
    Now includes full statistical analysis.
    """
    # Extract reaction times
    rt_with = [trial[1] for trial in results if trial[2] == "yes"]
    rt_without = [trial[1] for trial in results if trial[2] == "no"]

    # Compute statistics
    mean_rt_with = np.mean(rt_with) if rt_with else 0
    mean_rt_without = np.mean(rt_without) if rt_without else 0
    median_rt_with = np.median(rt_with) if rt_with else 0
    median_rt_without = np.median(rt_without) if rt_without else 0
    min_rt_with = np.min(rt_with) if rt_with else 0
    min_rt_without = np.min(rt_without) if rt_without else 0
    max_rt_with = np.max(rt_with) if rt_with else 0
    max_rt_without = np.max(rt_without) if rt_without else 0
    mean_diff = mean_rt_with - mean_rt_without
    median_diff = median_rt_with - median_rt_without

    # IQR Calculation (75th percentile - 25th percentile)
    iqr_rt_with = np.percentile(rt_with, 75) - np.percentile(rt_with, 25) if rt_with else 0
    iqr_rt_without = np.percentile(rt_without, 75) - np.percentile(rt_without, 25) if rt_without else 0

    # Mean Absolute Deviation (MAD)
    mad_rt_with = np.mean(np.abs(rt_with - np.mean(rt_with))) if rt_with else 0
    mad_rt_without = np.mean(np.abs(rt_without - np.mean(rt_without))) if rt_without else 0

    # Permutation Test p-values
    observed_diff_mean, p_value_mean = permutation_test_mean(rt_with, rt_without)
    observed_diff_median, p_value_median = permutation_test_median(rt_with, rt_without)

    # Convert distraction sequence to string for CSV
    distraction_sequence_str = ",".join(distraction_sequence)

    # === Save detailed results ===
    detailed_filename = generate_unique_filename("detailed_results")
    detailed_file_exists = os.path.isfile(detailed_filename)

    with open(detailed_filename, mode="a", newline="") as csv_file:
        writer = csv.writer(csv_file)
        if not detailed_file_exists:
            writer.writerow(["UserID", "Age", "Gender", "Trial", "Reaction_Time", "Distraction"])
        for trial in results:
            trial_num, reaction_time, distraction_flag = trial
            writer.writerow([user_id, age, gender, trial_num, reaction_time, distraction_flag])

    # === Save summary results ===
    summary_filename = generate_unique_filename("summary_results")
    summary_file_exists = os.path.isfile(summary_filename)

    with open(summary_filename, mode="a", newline="") as csv_file:
        writer = csv.writer(csv_file)
        if not summary_file_exists:
            writer.writerow([
                "UserID", "Age", "Gender", "Distraction_Sequence",
                "Mean_RT_with_Distract", "Mean_RT_without_Distract", "Mean_Diff",
                "Median_RT_with_Distract", "Median_RT_without_Distract", "Median_Diff",
                "Min_RT_with_Distract", "Min_RT_without_Distract",
                "Max_RT_with_Distract", "Max_RT_without_Distract",
                "IQR_RT_with_Distract", "IQR_RT_without_Distract",
                "MAD_with_Distract", "MAD_without_Distract",
                "p-value_Mean", "p-value_Median"
            ])
        writer.writerow([
            user_id, age, gender, distraction_sequence_str,
            f"{mean_rt_with:.2f}", f"{mean_rt_without:.2f}", f"{mean_diff:.2f}",
            f"{median_rt_with:.2f}", f"{median_rt_without:.2f}", f"{median_diff:.2f}",
            f"{min_rt_with:.2f}", f"{min_rt_without:.2f}",
            f"{max_rt_with:.2f}", f"{max_rt_without:.2f}",
            f"{iqr_rt_with:.2f}", f"{iqr_rt_without:.2f}",
            f"{mad_rt_with:.2f}", f"{mad_rt_without:.2f}",
            f"{p_value_mean:.6f}", f"{p_value_median:.6f}"
        ])

    print(Fore.LIGHTGREEN_EX + "Τα αποτελέσματα αποθηκεύτηκαν με επιτυχία!")


# Υπολογισμός Permutation Test για τη μέση διαφορά
def permutation_test_mean(data1, data2, num_permutations=20001):
    observed_diff = abs(np.mean(data1) - np.mean(data2))
    combined_data = np.concatenate([data1, data2])
    count_extreme = 0

    for _ in range(num_permutations):
        perm_data = combined_data.copy()  # Copy the data before shuffling
        np.random.shuffle(perm_data)
        perm_data1 = perm_data[:len(data1)]
        perm_data2 = perm_data[len(data1):]
        perm_diff = abs(np.mean(perm_data1) - np.mean(perm_data2))
        if perm_diff >= observed_diff:
            count_extreme += 1

    p_value = max(1 / num_permutations, count_extreme / num_permutations)  # Avoid p=0
    return observed_diff, p_value


# Υπολογισμός Permutation Test για τη διαφορά διαμέσου (median)
def permutation_test_median(data1, data2, num_permutations=30000):
    observed_diff = abs(np.median(data1) - np.median(data2))
    combined_data = np.concatenate([data1, data2])
    count_extreme = 0

    for _ in range(num_permutations):
        np.random.shuffle(combined_data)
        perm_data1 = combined_data[:len(data1)]
        perm_data2 = combined_data[len(data1):]
        perm_diff = abs(np.median(perm_data1) - np.median(perm_data2))
        if perm_data1.size > 0 and perm_data2.size > 0 and perm_diff >= observed_diff:
            count_extreme += 1

    p_value = max(1 / num_permutations, count_extreme / num_permutations)  # Avoid p=0
    return observed_diff, p_value

def main():
    while True:  # Allows restarting the experiment if the assistant rejects it
        print(WELCOME_MSG)
        print(INSTRUCTIONS)
        
        # Συλλογή στοιχείων συμμετέχοντα
        age = input(ASK_AGE)
        gender = input(ASK_GENDER)
        
        # Δημιουργία μοναδικού userID με milliseconds
        user_id = f"USER_{random.randint(0, 9999):04d}_{time.strftime('%Y%m%d_%H%M%S')}_{int((time.time()*1000) % 1000000):06d}"
        print(Fore.LIGHTCYAN_EX + f"\nΤο userID σας: {user_id}")
        
        # Εύρεση και σύνδεση με Arduino
        arduino_port = find_arduino()
        if not arduino_port:
            print(Fore.LIGHTRED_EX + "⚠️ Δεν βρέθηκε συσκευή Arduino. Ελέγξτε τη σύνδεση.")
            return
        
        try:
            arduino = serial.Serial(arduino_port, 9600, timeout=1)
            time.sleep(2)
        except Exception as e:
            print(Fore.LIGHTRED_EX + "⚠️ Σφάλμα σύνδεσης με Arduino:", e)
            return
        
        print(Fore.LIGHTCYAN_EX + "\nΤο πείραμα ξεκινά...")
        time.sleep(0.5)
        
        results = []

        # **Randomized assignment: exactly 5 distraction & 5 normal trials**
        distraction_sequence = ["yes"] * ROUNDS_PER_BLOCK + ["no"] * ROUNDS_PER_BLOCK
        random.shuffle(distraction_sequence)

        print(Fore.LIGHTRED_EX + f"\n🔹 Οι {2 * ROUNDS_PER_BLOCK} δοκιμασίες θα εκτελεστούν με τυχαία σειρά διασπαστικών και μη διασπαστικών γύρων.")
        
        for i in range(2 * ROUNDS_PER_BLOCK):
            distraction_flag = distraction_sequence[i]  # Assign distraction based on shuffled list
            results.append(run_trial(arduino, i + 1, distraction=(distraction_flag == "yes")))

        # 🚨 Assistant Confirmation Step 🚨
        while True:
            assistant_response = input(Fore.LIGHTMAGENTA_EX + "\n👤 Επιβεβαίωση Βοηθού: Ήταν το πείραμα έγκυρο; (Y/N): ").strip().upper()
            
            if assistant_response == "Y":
                print(Fore.LIGHTGREEN_EX + "✅ Το πείραμα επιβεβαιώθηκε. Τα αποτελέσματα θα αποθηκευτούν.")
                break  # Exit loop and save results
            elif assistant_response == "N":
                print(Fore.LIGHTRED_EX + "❌ Το πείραμα δεν ήταν έγκυρο! Θα επαναληφθεί από την αρχή.\n")
                arduino.close()
                return main()  # **Restart the entire function from the beginning**
            else:
                print(Fore.LIGHTYELLOW_EX + "⚠️ Μη έγκυρη απάντηση. Παρακαλώ πληκτρολογήστε 'Y' ή 'N'.")

        # 📌 Επεξεργασία και αποθήκευση αποτελεσμάτων
        process_results(user_id, age, gender, distraction_sequence, results)

        # Κλείσιμο της σύνδεσης με Arduino
        arduino.close()
        break  # Exit loop if assistant confirms the test



    # (Rest of the analysis and CSV writing follows...)
    # Διαχωρισμός των reaction times ανά κατάσταση
    rt_with = [trial[1] for trial in results if trial[2] == "yes" and trial[1] is not None]
    rt_without = [trial[1] for trial in results if trial[2] == "no" and trial[1] is not None]
    
    # Μέσες τιμές
    mean_rt_with = sum(rt_with) / len(rt_with) if rt_with else 0
    mean_rt_without = sum(rt_without) / len(rt_without) if rt_without else 0
    mean_diff = mean_rt_with - mean_rt_without
    
    # Διάμεσοι
    median_rt_with = np.median(rt_with) if rt_with else 0
    median_rt_without = np.median(rt_without) if rt_without else 0
    median_diff = median_rt_with - median_rt_without
    
    # IQR (75th percentile - 25th percentile)
    iqr_rt_with = np.percentile(rt_with, 75) - np.percentile(rt_with, 25) if rt_with else 0
    iqr_rt_without = np.percentile(rt_without, 75) - np.percentile(rt_without, 25) if rt_without else 0
    
    # Mean Absolute Deviation (from the mean)
    mean_abs_dev_rt_with = np.mean(np.abs(np.array(rt_with) - np.mean(rt_with))) if rt_with else 0
    mean_abs_dev_rt_without = np.mean(np.abs(np.array(rt_without) - np.mean(rt_without))) if rt_without else 0
    
    print(END_MSG)
    print(Fore.LIGHTCYAN_EX + "\n📊 Αποτελέσματα:")
    print(Fore.LIGHTBLUE_EX + f" - Μέσος χρόνος αντίδρασης με διασπαστικές ερωτήσεις: {mean_rt_with:.2f} ms")
    print(Fore.LIGHTYELLOW_EX + f" - Μέσος χρόνος αντίδρασης χωρίς διασπαστικές ερωτήσεις: {mean_rt_without:.2f} ms")
    print(Fore.LIGHTRED_EX + f" - Διαφορά μέσων χρόνων αντίδρασης: {mean_diff:.2f} ms")
    print(Fore.LIGHTBLUE_EX + f" - Mean Absolute Deviation με διασπαστικές ερωτήσεις: {mean_abs_dev_rt_with:.2f} ms")
    print(Fore.LIGHTYELLOW_EX + f" - Mean Absolute Deviation χωρίς διασπαστικές ερωτήσεις: {mean_abs_dev_rt_without:.2f} ms")
    print()
    print(Fore.LIGHTBLUE_EX + f" - Διάμεσος χρόνος με διασπαστικές ερωτήσεις: {median_rt_with:.2f} ms")
    print(Fore.LIGHTYELLOW_EX + f" - Διάμεσος χρόνος χωρίς διασπαστικές ερωτήσεις: {median_rt_without:.2f} ms")
    print(Fore.LIGHTRED_EX + f" - Διαφορά διάμεσων χρόνων αντίδρασης: {median_diff:.2f} ms")
    print(Fore.LIGHTBLUE_EX + f" - IQR με διασπαστικές ερωτήσεις: {iqr_rt_with:.2f} ms")
    print(Fore.LIGHTYELLOW_EX + f" - IQR χωρίς διασπαστικές ερωτήσεις: {iqr_rt_without:.2f} ms")
    print()
    print(Fore.LIGHTCYAN_EX + """
    ℹ️ Επεξήγηση στατιστικών:

    • 📏 IQR (Ενδοτεταρτημοριακό Εύρος): Δείχνει πόσο συγκεντρωμένοι είναι οι χρόνοι αντίδρασης στο μεσαίο 50% των τιμών.
      → Μικρό IQR = σταθερές αποκρίσεις. Μεγάλο IQR = μεγαλύτερη διακύμανση.

    • 📉 MAD (Μέση Απόλυτη Απόκλιση από τον Μέσο Όρο): Δείχνει πόσο απέχουν κατά μέσο όρο οι χρόνοι από τον συνολικό μέσο όρο.
      → Μικρό MAD = σταθερότητα. Μεγάλο MAD = οι αντιδράσεις διέφεραν σημαντικά.
    """)

    # Υποθέτουμε ότι rt_with και rt_without είναι οι λίστες των χρόνων αντίδρασης
    # rt_with = [..] και rt_without = [..] που έχουν ήδη υπολογιστεί στο πρόγραμμά σας

    mean_diff_observed = np.mean(rt_with) - np.mean(rt_without)
    median_diff = np.median(rt_with) - np.median(rt_without)

    combined_data = np.array(rt_with + rt_without)
    n = 50000  # αριθμός permutations
    mean_diffs = []
    median_diffs = []

    for _ in range(n):
        np.random.shuffle(combined_data)
        perm_with = combined_data[:len(rt_with)]
        perm_without = combined_data[len(rt_with):]

        mean_diffs.append(np.mean(perm_with) - np.mean(perm_without))
        median_diffs.append(np.median(perm_with) - np.median(perm_without))

    p_value_mean = np.mean(np.abs(mean_diffs) >= np.abs(mean_rt_with - mean_rt_without))
    p_value_median = np.mean(np.abs(median_diffs) >= np.abs(median_diff))

    
    # Υπολογισμός Permutation Test για τη μέση τιμή
    observed_diff_mean, p_value_mean = permutation_test_mean(np.array(rt_with), np.array(rt_without),n)

    print(Fore.LIGHTMAGENTA_EX + "\n🔍 Στατιστικός Έλεγχος με Permutation Test (Μέσος Χρόνος):")
    print(Fore.LIGHTBLUE_EX + f"Παρατηρούμενη διαφορά μέσων χρόνων: {observed_diff_mean:.2f} ms")
    print(Fore.LIGHTMAGENTA_EX + f"Permutation test p-value: {p_value_mean:.6f}")

    if p_value_mean < 0.001:
        print(Fore.LIGHTYELLOW_EX + f"🔴 Η πιθανότητα να παρατηρηθεί μια τέτοια διαφορά κατά τύχη ή λόγω άλλων άγνωστων παραγόντων, δηλ. χωρίς να επηρεάζουν οι διασπαστικές ερωτήσεις, είναι εξαιρετικά μικρή ({p_value_mean * 100:.4f}%).")
        print(Fore.LIGHTRED_EX + "🔴 Υπάρχει πολύ ισχυρή στατιστική ένδειξη (p < 0.1 %) ότι οι διασπαστικές ερωτήσεις επηρεάζουν σημαντικά τον χρόνο αντίδρασης.")
    elif p_value_mean < 0.05:
        print(Fore.LIGHTYELLOW_EX + f"🟠 Η πιθανότητα να παρατηρηθεί μια τέτοια διαφορά κατά τύχη ή λόγω άλλων άγνωστων παραγόντων, δηλ. χωρίς να επηρεάζουνν οι διασπαστικές ερωτήσεις, είναι {p_value_mean * 100:.2f}%.")
        print(Fore.LIGHTRED_EX + "🔴 Υπάρχει στατιστικά σημαντική διαφορά (p < 5 %). Οι διασπαστικές ερωτήσεις επηρεάζουν σημαντικά τον χρόνο αντίδρασης.")
    else:
        print(Fore.LIGHTGREEN_EX + f"🟢 Δεν υπάρχει στατιστικά σημαντική διαφορά (p = {p_value_mean:.4f} ≥ 0.05).")


    # Υπολογισμός Permutation Test για τη διάμεσο
    observed_diff_median, p_value_median = permutation_test_median(np.array(rt_with), np.array(rt_without),n)

    print(Fore.LIGHTMAGENTA_EX + "\n🔍 Στατιστικός Έλεγχος με Permutation Test (Διάμεσος Χρόνος):")
    print(Fore.LIGHTBLUE_EX + f"Παρατηρούμενη διαφορά διαμέσου: {observed_diff_median:.2f} ms")
    print(Fore.LIGHTMAGENTA_EX + f"Permutation test p-value: {p_value_median:.6f}")


    if p_value_median < 0.001:
        print(Fore.LIGHTYELLOW_EX + f"🔴 Η πιθανότητα να παρατηρηθεί μια τέτοια διαφορά στη διάμεσο κατά τύχη ή λόγω άλλων άγνωστων παραγόντων, δηλ. χωρίς να επηρεάζουν οι διασπαστικές ερωτήσεις, είναι εξαιρετικά μικρή ({p_value_median * 100:.4f}%).")
        print(Fore.LIGHTRED_EX + "🔴 Υπάρχει πολύ ισχυρή στατιστική ένδειξη (p < 0.1 %) ότι οι διασπαστικές ερωτήσεις επηρεάζουν σημαντικά τη διάμεσο του χρόνου αντίδρασης.")
    elif p_value_median < 0.05:
        print(Fore.LIGHTYELLOW_EX + f"🟠 Η πιθανότητα να παρατηρηθεί μια τέτοια διαφορά στη διάμεσο κατά τύχη ή λόγω άλλων άγνωστων παραγόντων είναι {p_value_median * 100:.2f}%.")
        print(Fore.LIGHTRED_EX + "🔴 Υπάρχει στατιστικά σημαντική διαφορά (p < 5 %). Οι διασπαστικές ερωτήσεις επηρεάζουν σημαντικά τη διάμεσο του χρόνου αντίδρασης.")
    else:
        print(Fore.LIGHTGREEN_EX + f"🟢 Δεν υπάρχει στατιστικά σημαντική διαφορά στη διάμεσο (p = {p_value_median:.4f} ≥ 0.05).")


    print(Fore.LIGHTCYAN_EX + "\n📢 Πώς να ερμηνεύσετε τα αποτελέσματα του πειράματος;")

    if p_value_mean < 0.05 and p_value_median < 0.05:
        print(Fore.LIGHTRED_EX + "🔴 Και ο μέσος όρος και η διάμεσος έχουν στατιστικά σημαντική διαφορά.")
        print("✅ Οι διασπαστικές ερωτήσεις επηρέασαν ΙΣΧΥΡΑ και ΣΤΑΘΕΡΑ τις αντιδράσεις σας.")

    elif p_value_mean < 0.05 and p_value_median >= 0.05:
        print(Fore.LIGHTYELLOW_EX + "⚠️ Μόνο ο μέσος όρος είχε στατιστικά σημαντική διαφορά.")
        print("🔍 Υπήρξαν μεμονωμένες μεγάλες καθυστερήσεις, όχι όμως γενικευμένη επίδραση.")

    elif p_value_mean >= 0.05 and p_value_median < 0.05:
        print(Fore.LIGHTYELLOW_EX + "⚠️ Μόνο η διάμεσος έχει στατιστικά σημαντική διαφορά.")
        print("🔍 Οι αντιδράσεις σας ήταν σταθερά πιο αργές, αλλά χωρίς ακραίες τιμές.")

    else:
        print(Fore.LIGHTGREEN_EX + "🟢 Καμία στατιστικά σημαντική διαφορά δεν εντοπίστηκε.")
        print("💡 Οι διασπαστικές ερωτήσεις δεν είχαν ουσιαστική επίδραση στις αντιδράσεις σας.")

    
    print(Fore.LIGHTCYAN_EX + "\nΣας ευχαριστούμε για τη συμμετοχή σας!")

if __name__ == "__main__":
    main()


